package game;
import java.awt.*;

public class MovingThing extends Thing
{
    private final double
        MAX_SPEED = getMaxSpeed();
    protected final double
        BRAKE_AT = getBrakeAt(),
        BRAKE_ADJUST_SPEED = getBrakeAdjustSpeed();

    private Direction direction = new Direction();

    private int n;
    protected double speedX = 0, speedY = 0;
    private shape.Size marginForBorder = new shape.Size(0,0);

    public MovingThing(int _width, int _height, int red, int green, int blue)
    {
        super(_width, _height, red, green, blue);
    }
    public MovingThing(shape.Size _size, int red, int green, int blue)
    {
        super(_size, red, green, blue);
    }
    public MovingThing(int _width, int _height)
    {
        super(_width, _height);
    }

    public int getFacing()
    {
        return direction.getFacing(speedX, speedY);
    }

    public shape.Point getPoint()
    {
        return new shape.Point(x,y);
    }

    public int getLeft()
    {
        return (int) x;
    }
    public int getTop()
    {
        return (int) y;
    }
    public int getRight()
    {
        return (int) x+size.width;
    }
    public int getBottom()
    {
        return (int) y+size.height;
    }

    public shape.Rectangle getRectangle()
    {
        return new shape.Rectangle( x, y, x+size.width, y+size.height );
    }

    public int getMidOff()
    {
        return (int) (size.width+size.height)/4;
    }

    public double getSpeedX()
    {
        return speedX;
    }
    public double getSpeedY()
    {
        return speedY;
    }

    public void randSpeed(boolean randX, boolean randY)
    {
        if ( randX )
            speedX = (int) ( Math.random()* 5 );
        if ( randY )
            speedY = (int) ( Math.random()* 5 );
    }

    public void push(double pushX, double pushY)
    {
        if ( speedX+pushX >= -MAX_SPEED &&
             speedX+pushX <=  MAX_SPEED
           )
            speedX += pushX;

        if ( speedY+pushY >= -MAX_SPEED &&
             speedY+pushY <=  MAX_SPEED
           )
            speedY += pushY;
    }

    protected double getMaxSpeed()    
    {
        return 7;
    }

    public void push(int pushX, int pushY)
    {
        push( (double) pushX, (double) pushY );
    }

    protected double getBrakeAdjustSpeed()
    {
        return 2;
    }

    protected double getBrakeAt()
    {
        return .2;
    }

    public RectState positionState(Ground grnd)
    {
        RectState rectState = new RectState();
        rectState.checkState( grnd.getRectangle(), getMarginRectangle() );
        return rectState;
    }

    public void shift(Ground grnd)
    {
        RectState rectState = new RectState();
        rectState.checkState( grnd.getRectangle(), getRectangle() );

        if ( rectState.getIsOutLeft() )
            x = grnd.right-size.width;
        else if ( rectState.getIsOutRight() )
            x = grnd.left;

        if ( rectState.getIsOutTop() )
            y = grnd.bottom-size.height;
        else if ( rectState.getIsOutBottom() )
            y = grnd.top;
    }

    public boolean shiftDisappear(Ground grnd)
    {
        boolean rVal = false;
        RectState rectState = new RectState();

        rectState.checkStateDisappear( grnd.getRectangle(), getMarginRectangle() );

        if ( rectState.getIsOutLeft() )
        {
            x = grnd.right;
            rVal = true;
        }
        else if ( rectState.getIsOutRight() )
        {
            x = grnd.left-size.width;
            rVal = true;
        }

        if ( rectState.getIsOutTop() )
        {
            y = grnd.bottom-size.height;
            rVal = true;
        }            
        else if ( rectState.getIsOutBottom() )
        {
            y = grnd.top;
             rVal = true;
        }

        return rVal;
    }

    public void setMarginForBorder(shape.Size _marginForBorder)
    {
        marginForBorder.width = _marginForBorder.width;
        marginForBorder.height = _marginForBorder.height;
    }

    public void bounce(Ground grnd)
    {
        positionKeepBorder(grnd, true);
    }

    public void bump(Ground grnd)
    {
        positionKeepBorder(grnd, false);
    }

    private void positionKeepBorder(Ground grnd, boolean bounceOff)
    {
        RectState rectState = new RectState();
        rectState.checkState( grnd.getRectangle(), getMarginRectangle() );
        position( keepBorders(grnd, rectState, getPoint(), bounceOff) );
    }

    private shape.Rectangle getMarginRectangle()
    {
        shape.Rectangle marginRectangle = getRectangle();
        marginRectangle.addMargin(marginForBorder);
        return marginRectangle;
    }

    private shape.Point keepBorders(Ground ground, RectState rectState, shape.Point currentPoint, boolean bounceOff)
    {
        final boolean doRePosition = false;
        shape.Point returnPoint = new shape.Point( currentPoint );
        boolean hitX = false, hitY = false;

        if ( rectState.isOutHorizontal() )
        {
            hitX = rectState.getIsOutLeft() ? speedX < 0 : speedX > 0;

            if ( rectState.getIsOutLeft()  )
            {
                hitX = speedX < 0;
                if ( doRePosition )
                    returnPoint.x = ground.left+marginForBorder.width;
            }
            else // if ( rectState.getIsOutRight() )
            {
                hitX = speedX > 0;
                if ( doRePosition )
                    returnPoint.x = ground.right-(size.width+marginForBorder.width);
            }

        }
        if ( rectState.isOutVertical() )
        {
            if ( rectState.getIsOutTop() )
            {
                hitY = speedY < 0;
                if ( doRePosition )
                    returnPoint.y = ground.top+marginForBorder.height;
            }
            else // if ( rectState.getIsOutBottom() )
            {
                hitY = speedY > 0;
                if ( doRePosition )
                    returnPoint.y = ground.bottom-(size.height+marginForBorder.height);
            }
        }

        if ( hitX )
            speedX = doAdjustSpeed(speedX, bounceOff);
        if ( hitY )
            speedY = doAdjustSpeed(speedY, bounceOff);

        return returnPoint;
    }

    private double doAdjustSpeed(double pos, boolean bounceOff)
    {
        return bounceOff ? speedX *= -1 : 0;
    }

    public void move(Ground grnd)
    {
        x += speedX;
        y += speedY;
    }

    public shape.Point getAppearPoint(Ground ground, Camera camera)
    {
        return new shape.Point(
            getAppearX(ground, camera), getAppearY(ground, camera)
            );
    }

    public int number()
    { // number of object
        return n;
    }

    public String getDebugInfo()
    {
        return
            "x: "+ x +" | y: "+ y +
            " | speedX: "+ speedX +" | speedY: "+ speedY;
    }

    protected void slowDownX()
    {
        speedX = slowDown(speedX);
    }

    protected void slowDownY()
    {
        speedY = slowDown(speedY);
    }

    protected double slowDown(double pos)
    {
        if ( pos < 0 )
        {
            pos += BRAKE_ADJUST_SPEED;
            if ( pos >= -BRAKE_AT )
                pos = 0;
        }
        else if ( pos > 0 )
        {
            pos -= BRAKE_ADJUST_SPEED;
            if ( pos <= BRAKE_AT )
                pos = 0;
        }
        return pos;
    }

}