package game;
import java.awt.*;

public class Player extends MovingThing
{
    private art.ShipArt shipArt = new art.ShipArt();
    private final int SHOOT_DELAY = 1;
    private int shootDelayValue = SHOOT_DELAY;
    private final static double PUSH_STRENGTH = 5;

    public Player()
    {
        super (32,32);
    }

    public void act(Ground ground, Input input, Rockets rocket)
    {
        handleInput(input, rocket);
        move(ground);
    }

    protected double getBrakeAt()
    {
        return .8;
    }
    protected double getBrakeAdjustSpeed()
    {
        return 2;
    }
    protected double getMaxSpeed()
    {
        return 20;
    }

    public void appear(Graphics graph, Ground ground, Camera camera)
    {
        shipArt.draw( graph, getAppearPoint(ground, camera), size, getFacing() );
    }

    private void handleInput(Input input, Rockets rockets)
    {
        switch ( input.getInput() )
        {
           case Input.UP:
                push( 0, -PUSH_STRENGTH );
                shipArt.setAction( shipArt.ACTION_SPEEDING );
                slowDownX();
                break;
           case Input.RIGHT:
                push( PUSH_STRENGTH, 0 );
                shipArt.setAction( shipArt.ACTION_SPEEDING );
                slowDownY();
                break;
           case Input.DOWN:
                push( 0, PUSH_STRENGTH );
                shipArt.setAction( shipArt.ACTION_SPEEDING );
                slowDownX();
                break;
           case Input.LEFT:
                push( -PUSH_STRENGTH, 0 );
                shipArt.setAction( shipArt.ACTION_SPEEDING );
                slowDownY();
                break;

           case Input.ACTION:
                if ( shootDelayValue == 0 )
                {
                    shipArt.setAction( shipArt.ACTION_ATTACKING );
                    shootRockets(rockets);
                    slowDownX();
                    slowDownY();
                    shootDelayValue = SHOOT_DELAY;
                }
                break;

           default:
                if ( shootDelayValue > 0 )
                    shootDelayValue--;
                shipArt.setAction( shipArt.ACTION_UNDEFINED );
                slowDownX();
                slowDownY();
                break;
        }
    }

    private void shootRockets(Rockets rockets)
    {
        int gunFacing = getFacing();
        rockets.createOne( getLeftGunPoint(gunFacing), gunFacing );
        rockets.createOne( getRightGunPoint(gunFacing), gunFacing );
    }

    private shape.Point getLeftGunPoint(int facing)
    {
        double gunX = 0, gunY = 0;
        switch (facing)
        {
            case Directions.FACING_UP:
                gunX = x;
                gunY = y + size.height/2 - Rocket.HEIGHT;
                break;
            case Directions.FACING_RIGHT:
                gunX = x + size.width/2;
                gunY = y - Rocket.WIDTH;
                break;
            case Directions.FACING_DOWN:
                gunX = x + size.width - Rocket.WIDTH;
                gunY = y + size.height/2;
                break;
            case Directions.FACING_LEFT:
                gunX = x + size.width/2;
                gunY = y + size.height - Rocket.HEIGHT;
                break;
        }
        return new shape.Point( gunX, gunY );
    }
    private shape.Point getRightGunPoint(int facing)
    {
        double gunX = 0, gunY = 0;
        switch (facing)
        {
            case Directions.FACING_UP:
                gunX = x + size.width - Rocket.WIDTH;
                gunY = y + size.height/2 - Rocket.HEIGHT;
                break;
            case Directions.FACING_RIGHT:
                gunX = x + size.width/2;
                gunY = y + size.height - Rocket.HEIGHT;
                break;
            case Directions.FACING_DOWN:
                gunX = x;
                gunY = y + size.height/2;
                break;
            case Directions.FACING_LEFT:
                gunX = x + size.width/2;
                gunY = y - Rocket.WIDTH;
                break;
        }
        return new shape.Point( gunX, gunY );
    }
}